@@ -1,8 +1,8 @@
 /* LIBLDAP url.c -- LDAP URL (RFC 4516) related routines */
-/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.94.2.8 2008/02/11 23:41:37 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2008 The OpenLDAP Foundation.
+ * Copyright 1998-2015 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -590,9 +590,11 @@ desc2str_len( LDAPURLDesc *u )
 	}
 
 	if ( u->lud_host && u->lud_host[0] ) {
+		char *ptr;
 		len += hex_escape_len( u->lud_host, URLESC_SLASH );
-		if ( !is_ipc && strchr( u->lud_host, ':' )) {
-			len += 2;	/* IPv6, [] */
+		if ( !is_ipc && ( ptr = strchr( u->lud_host, ':' ))) {
+			if ( strchr( ptr+1, ':' ))
+				len += 2;	/* IPv6, [] */
 		}
 	}
 
@@ -610,6 +612,7 @@ desc2str( LDAPURLDesc *u, char *s, int len )
 	int		is_v6 = 0;
 	int		is_ipc = 0;
 	struct berval	scope = BER_BVNULL;
+	char		*ptr;
 
 	if ( u == NULL ) {
 		return -1;
@@ -637,8 +640,9 @@ desc2str( LDAPURLDesc *u, char *s, int len )
 		sep = 1;
 	}
 
-	if ( !is_ipc && u->lud_host && strchr( u->lud_host, ':' )) {
-		is_v6 = 1;
+	if ( !is_ipc && u->lud_host && ( ptr = strchr( u->lud_host, ':' ))) {
+		if ( strchr( ptr+1, ':' ))
+			is_v6 = 1;
 	}
 
 	if ( u->lud_port ) {
@@ -1421,6 +1425,7 @@ ldap_url_list2hosts (LDAPURLDesc *ludlist)
 	/* figure out how big the string is */
 	size = 1;	/* nul-term */
 	for (ludp = ludlist; ludp != NULL; ludp = ludp->lud_next) {
+		if ( ludp->lud_host == NULL ) continue;
 		size += strlen(ludp->lud_host) + 1;		/* host and space */
 		if (strchr(ludp->lud_host, ':'))        /* will add [ ] below */
 			size += 2;
@@ -1433,6 +1438,7 @@ ldap_url_list2hosts (LDAPURLDesc *ludlist)
 
 	p = s;
 	for (ludp = ludlist; ludp != NULL; ludp = ludp->lud_next) {
+		if ( ludp->lud_host == NULL ) continue;
 		if (strchr(ludp->lud_host, ':')) {
 			p += sprintf(p, "[%s]", ludp->lud_host);
 		} else {
@@ -1445,7 +1451,7 @@ ldap_url_list2hosts (LDAPURLDesc *ludlist)
 	}
 	if (p != s)
 		p--;	/* nuke that extra space */
-	*p = 0;
+	*p = '\0';
 	return s;
 }
 
