@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2015 The OpenLDAP Foundation.
  * Portions Copyright 1999-2003 Howard Chu.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * All rights reserved.
@@ -75,10 +75,6 @@ ldap_back_modify(
 		mods[ i ].mod_type = ml->sml_desc->ad_cname.bv_val;
 
 		if ( ml->sml_values != NULL ) {
-			if ( ml->sml_values == NULL ) {	
-				continue;
-			}
-
 			for ( j = 0; !BER_BVISNULL( &ml->sml_values[ j ] ); j++ )
 				/* just count mods */ ;
 			mods[ i ].mod_bvalues =
@@ -102,7 +98,6 @@ retry:;
 	rc = ldap_back_controls_add( op, rs, lc, &ctrls );
 	if ( rc != LDAP_SUCCESS ) {
 		send_ldap_result( op, rs );
-		rc = -1;
 		goto cleanup;
 	}
 
@@ -120,6 +115,10 @@ retry:;
 		}
 	}
 
+	ldap_pvt_thread_mutex_lock( &li->li_counter_mutex );
+	ldap_pvt_mp_add( li->li_ops_completed[ SLAP_OP_MODIFY ], 1 );
+	ldap_pvt_thread_mutex_unlock( &li->li_counter_mutex );
+
 cleanup:;
 	(void)ldap_back_controls_free( op, rs, &ctrls );
 
@@ -132,6 +131,6 @@ cleanup:;
 		ldap_back_release_conn( li, lc );
 	}
 
-	return rc;
+	return rs->sr_err;
 }
 
