@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2015 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * Portions Copyright 1999-2003 Howard Chu.
  * All rights reserved.
@@ -69,12 +69,14 @@ struct ldaprwmap {
 	 * will be disabled */
 	BerVarray rwm_suffix_massage;
 #endif /* !ENABLE_REWRITE */
+	BerVarray rwm_bva_rewrite;
 
 	/*
 	 * Attribute/objectClass mapping
 	 */
 	struct ldapmap rwm_oc;
 	struct ldapmap rwm_at;
+	BerVarray rwm_bva_map;
 };
 
 /* Whatever context ldap_back_dn_massage needs... */
@@ -124,14 +126,6 @@ ldap_back_map_attrs(
 	int remap,
 	char ***mapped_attrs );
 
-extern int ldap_back_map_config(
-	struct ldapmap	*oc_map,
-	struct ldapmap	*at_map,
-	const char	*fname,
-	int		lineno,
-	int		argc,
-	char		**argv );
-
 extern int
 ldap_back_filter_map_rewrite(
 	dncookie	*dc,
@@ -273,8 +267,8 @@ typedef struct metasubtree_t {
 	union {
 		struct berval msu_dn;
 		struct {
+			struct berval msr_regex_pattern;
 			regex_t msr_regex;
-			char *msr_regex_pattern;
 		} msu_regex;
 	} ms_un;
 #define ms_dn ms_un.msu_dn
@@ -284,6 +278,48 @@ typedef struct metasubtree_t {
 	struct metasubtree_t *ms_next;
 } metasubtree_t;
 
+typedef struct metafilter_t {
+	struct metafilter_t *mf_next;
+	struct berval mf_regex_pattern;
+	regex_t mf_regex;
+} metafilter_t;
+
+typedef struct metacommon_t {
+	int				mc_version;
+	int				mc_nretries;
+#define META_RETRY_UNDEFINED	(-2)
+#define META_RETRY_FOREVER	(-1)
+#define META_RETRY_NEVER	(0)
+#define META_RETRY_DEFAULT	(10)
+
+	unsigned		mc_flags;
+#define	META_BACK_CMN_ISSET(mc,f)		( ( (mc)->mc_flags & (f) ) == (f) )
+#define	META_BACK_CMN_QUARANTINE(mc)		META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_QUARANTINE )
+#define	META_BACK_CMN_CHASE_REFERRALS(mc)	META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_CHASE_REFERRALS )
+#define	META_BACK_CMN_NOREFS(mc)		META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_NOREFS )
+#define	META_BACK_CMN_NOUNDEFFILTER(mc)		META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_NOUNDEFFILTER )
+#define	META_BACK_CMN_SAVECRED(mc)		META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_SAVECRED )
+#define	META_BACK_CMN_ST_REQUEST(mc)		META_BACK_CMN_ISSET( (mc), LDAP_BACK_F_ST_REQUEST )
+
+#ifdef SLAPD_META_CLIENT_PR
+	/*
+	 * client-side paged results:
+	 * -1: accept unsolicited paged results responses
+	 *  0: off
+	 * >0: always request paged results with size == mt_ps
+	 */
+#define META_CLIENT_PR_DISABLE			(0)
+#define META_CLIENT_PR_ACCEPT_UNSOLICITED	(-1)
+	ber_int_t		mc_ps;
+#endif /* SLAPD_META_CLIENT_PR */
+
+	slap_retry_info_t	mc_quarantine;
+	time_t			mc_network_timeout;
+	struct timeval	mc_bind_timeout;
+#define META_BIND_TIMEOUT	LDAP_BACK_RESULT_UTIMEOUT
+	time_t			mc_timeout[ SLAP_OP_LAST ];
+} metacommon_t;
+
 typedef struct metatarget_t {
 	char			*mt_uri;
 	ldap_pvt_thread_mutex_t	mt_uri_mutex;
@@ -294,6 +330,7 @@ typedef struct metatarget_t {
 	LDAP_URLLIST_PROC	*mt_urllist_f;
 	void			*mt_urllist_p;
 
+	metafilter_t	*mt_filter;
 	metasubtree_t		*mt_subtree;
 	/* F: subtree-include; T: subtree-exclude */
 	int			mt_subtree_exclude;
@@ -323,19 +360,21 @@ typedef struct metatarget_t {
 #define	mt_idassert_flags	mt_idassert.si_flags
 #define	mt_idassert_authz	mt_idassert.si_authz
 
-	int			mt_nretries;
-#define META_RETRY_UNDEFINED	(-2)
-#define META_RETRY_FOREVER	(-1)
-#define META_RETRY_NEVER	(0)
-#define META_RETRY_DEFAULT	(10)
-
 	struct ldaprwmap	mt_rwmap;
 
 	sig_atomic_t		mt_isquarantined;
-	slap_retry_info_t	mt_quarantine;
 	ldap_pvt_thread_mutex_t	mt_quarantine_mutex;
 
-	unsigned		mt_flags;
+	metacommon_t	mt_mc;
+#define	mt_nretries	mt_mc.mc_nretries
+#define	mt_flags	mt_mc.mc_flags
+#define	mt_version	mt_mc.mc_version
+#define	mt_ps		mt_mc.mc_ps
+#define	mt_network_timeout	mt_mc.mc_network_timeout
+#define	mt_bind_timeout	mt_mc.mc_bind_timeout
+#define	mt_timeout	mt_mc.mc_timeout
+#define	mt_quarantine	mt_mc.mc_quarantine
+
 #define	META_BACK_TGT_ISSET(mt,f)		( ( (mt)->mt_flags & (f) ) == (f) )
 #define	META_BACK_TGT_ISMASK(mt,m,f)		( ( (mt)->mt_flags & (m) ) == (f) )
 
@@ -366,24 +405,6 @@ typedef struct metatarget_t {
 
 	slap_mask_t		mt_rep_flags;
 
-	int			mt_version;
-
-#ifdef SLAPD_META_CLIENT_PR
-	/*
-	 * client-side paged results:
-	 * -1: accept unsolicited paged results responses
-	 *  0: off
-	 * >0: always request paged results with size == mt_ps
-	 */
-#define META_CLIENT_PR_DISABLE			(0)
-#define META_CLIENT_PR_ACCEPT_UNSOLICITED	(-1)
-	ber_int_t		mt_ps;
-#endif /* SLAPD_META_CLIENT_PR */
-
-	time_t			mt_network_timeout;
-	struct timeval		mt_bind_timeout;
-#define META_BIND_TIMEOUT	LDAP_BACK_RESULT_UTIMEOUT
-	time_t			mt_timeout[ SLAP_OP_LAST ];
 } metatarget_t;
 
 typedef struct metadncache_t {
@@ -409,7 +430,15 @@ typedef struct metainfo_t {
 	int			mi_ntargets;
 	int			mi_defaulttarget;
 #define META_DEFAULT_TARGET_NONE	(-1)
-	int			mi_nretries;
+
+#define	mi_nretries	mi_mc.mc_nretries
+#define	mi_flags	mi_mc.mc_flags
+#define	mi_version	mi_mc.mc_version
+#define	mi_ps		mi_mc.mc_ps
+#define	mi_network_timeout	mi_mc.mc_network_timeout
+#define	mi_bind_timeout	mi_mc.mc_bind_timeout
+#define	mi_timeout	mi_mc.mc_timeout
+#define	mi_quarantine	mi_mc.mc_quarantine
 
 	metatarget_t		**mi_targets;
 	metacandidates_t	*mi_candidates;
@@ -429,14 +458,12 @@ typedef struct metainfo_t {
 	int			mi_conn_priv_max;
 
 	/* NOTE: quarantine uses the connection mutex */
-	slap_retry_info_t	mi_quarantine;
 	meta_back_quarantine_f	mi_quarantine_f;
 	void			*mi_quarantine_p;
 
-	unsigned		mi_flags;
 #define	li_flags		mi_flags
 /* uses flags as defined in <back-ldap/back-ldap.h> */
-#define	META_BACK_F_ONERR_STOP		(0x01000000U)
+#define	META_BACK_F_ONERR_STOP		LDAP_BACK_F_ONERR_STOP
 #define	META_BACK_F_ONERR_REPORT	(0x02000000U)
 #define	META_BACK_F_ONERR_MASK		(META_BACK_F_ONERR_STOP|META_BACK_F_ONERR_REPORT)
 #define	META_BACK_F_DEFER_ROOTDN_BIND	(0x04000000U)
@@ -455,19 +482,10 @@ typedef struct metainfo_t {
 
 #define META_BACK_QUARANTINE(mi)	LDAP_BACK_ISSET( (mi), LDAP_BACK_F_QUARANTINE )
 
-	int			mi_version;
-
-#ifdef SLAPD_META_CLIENT_PR
-	ber_int_t		mi_ps;
-#endif /* SLAPD_META_CLIENT_PR */
-
-
-	time_t			mi_network_timeout;
 	time_t			mi_conn_ttl;
 	time_t			mi_idle_timeout;
-	struct timeval		mi_bind_timeout;
-	time_t			mi_timeout[ SLAP_OP_LAST ];
 
+	metacommon_t	mi_mc;
 	ldap_extra_t	*mi_ldap_extra;
 
 } metainfo_t;
@@ -664,9 +682,20 @@ meta_dncache_delete_entry(
 extern void
 meta_dncache_free( void *entry );
 
+extern void
+meta_back_map_free( struct ldapmap *lm );
+
 extern int
 meta_subtree_destroy( metasubtree_t *ms );
 
+extern void
+meta_filter_destroy( metafilter_t *mf );
+
+extern int
+meta_target_finish( metainfo_t *mi, metatarget_t *mt,
+	const char *log, char *msg, size_t msize
+);
+
 extern LDAP_REBIND_PROC		meta_back_default_rebind;
 extern LDAP_URLLIST_PROC	meta_back_default_urllist;
 
