@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapcat.c,v 1.7.2.2 2007/08/31 23:14:00 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2015 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
@@ -32,7 +32,7 @@
 #include "slapcommon.h"
 #include "ldif.h"
 
-static int gotsig;
+static volatile sig_atomic_t gotsig;
 
 static RETSIGTYPE
 slapcat_sig( int sig )
@@ -47,9 +47,13 @@ slapcat( int argc, char **argv )
 	int rc = EXIT_SUCCESS;
 	Operation op = {0};
 	const char *progname = "slapcat";
+	int requestBSF;
+	int doBSF = 0;
 
 	slap_tool_init( progname, SLAPCAT, argc, argv );
 
+	requestBSF = ( sub_ndn.bv_len || filter );
+
 #ifdef SIGPIPE
 	(void) SIGNAL( SIGPIPE, slapcat_sig );
 #endif
@@ -61,7 +65,7 @@ slapcat( int argc, char **argv )
 
 	if( !be->be_entry_open ||
 		!be->be_entry_close ||
-		!be->be_entry_first ||
+		!( be->be_entry_first_x || be->be_entry_first ) ||
 		!be->be_entry_next ||
 		!be->be_entry_get )
 	{
@@ -77,9 +81,22 @@ slapcat( int argc, char **argv )
 	}
 
 	op.o_bd = be;
-	for ( id = be->be_entry_first( be );
-		id != NOID;
-		id = be->be_entry_next( be ) )
+	if ( !requestBSF && be->be_entry_first ) {
+		id = be->be_entry_first( be );
+
+	} else {
+		if ( be->be_entry_first_x ) {
+			id = be->be_entry_first_x( be,
+				sub_ndn.bv_len ? &sub_ndn : NULL, scope, filter );
+
+		} else {
+			assert( be->be_entry_first != NULL );
+			doBSF = 1;
+			id = be->be_entry_first( be );
+		}
+	}
+
+	for ( ; id != NOID; id = be->be_entry_next( be ) )
 	{
 		char *data;
 		int len;
@@ -92,28 +109,46 @@ slapcat( int argc, char **argv )
 		if ( e == NULL ) {
 			printf("# no data for entry id=%08lx\n\n", (long) id );
 			rc = EXIT_FAILURE;
-			if( continuemode ) continue;
-			break;
+			if ( continuemode == 0 ) {
+				break;
+
+			} else if ( continuemode == 1 ) {
+				continue;
+			}
+
+			/* this is a last resort: linearly scan all ids
+			 * trying to recover as much as possible (ITS#6482) */
+			while ( ++id != NOID ) {
+				e = be->be_entry_get( be, id );
+				if ( e != NULL ) break;
+				printf("# no data for entry id=%08lx\n\n", (long) id );
+			}
+
+			if ( e == NULL ) break;
 		}
 
-		if( sub_ndn.bv_len && !dnIsSuffix( &e->e_nname, &sub_ndn ) ) {
-			be_entry_release_r( &op, e );
-			continue;
-		}
-
-		if( filter != NULL ) {
-			int rc = test_filter( NULL, e, filter );
-			if( rc != LDAP_COMPARE_TRUE ) {
+		if ( doBSF ) {
+			if ( sub_ndn.bv_len && !dnIsSuffixScope( &e->e_nname, &sub_ndn, scope ) )
+			{
 				be_entry_release_r( &op, e );
 				continue;
 			}
+
+
+			if ( filter != NULL ) {
+				int rc = test_filter( NULL, e, filter );
+				if ( rc != LDAP_COMPARE_TRUE ) {
+					be_entry_release_r( &op, e );
+					continue;
+				}
+			}
 		}
 
-		if( verbose ) {
+		if ( verbose ) {
 			printf( "# id=%08lx\n", (long) id );
 		}
 
-		data = entry2str( e, &len );
+		data = entry2str_wrap( e, &len, ldif_wrap );
 		be_entry_release_r( &op, e );
 
 		if ( data == NULL ) {
@@ -123,12 +158,18 @@ slapcat( int argc, char **argv )
 			break;
 		}
 
-		fputs( data, ldiffp->fp );
-		fputs( "\n", ldiffp->fp );
+		if ( fputs( data, ldiffp->fp ) == EOF ||
+			fputs( "\n", ldiffp->fp ) == EOF ) {
+			fprintf(stderr, "%s: error writing output.\n",
+				progname);
+			rc = EXIT_FAILURE;
+			break;
+		}
 	}
 
 	be->be_entry_close( be );
 
-	slap_tool_destroy();
+	if ( slap_tool_destroy())
+		rc = EXIT_FAILURE;
 	return rc;
 }
